<?php
/* HTMLConverter - Simple HTML to PDF/RTF/TXT/etc converter
 * Copyright 2004-2006, Steve Blinch
 * http://code.blitzaffe.com
 * ============================================================================
 *
 * LICENSE
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */ 

if (is_dir($pdfpath = dirname(__FILE__).'/pdf/')) {
	define('PDF_PATH',$pdfpath);
} elseif (file_exists($pdfpath = dirname(__FILE__).'/../pdf/')) {
	define('PDF_PATH',$pdfpath);
}

if (!defined("FONT_PATH")) define("FONT_PATH",PDF_PATH."fonts/");


require_once(dirname(__FILE__)."/class_DocumentMaker.php");
require_once(dirname(__FILE__)."/class_CezpdfExtended.php");

class PDFMaker extends DocumentMaker {
	
	function PDFMaker() {
		unset($this->pdf);
	}
	
	function generator_options($options) {
	}

	function document_begin($orientation="portrait",$paperformat="a4") {
		
		$this->pdf =& new CezpdfExtended($paperformat,$orientation);
		
		$this->dpi = 72;
		
		$this->INDENT_INCREMENT = $this->indent_inches * $this->dpi;

		$this->page_width = $this->pdf->ez['pageWidth'];
		$this->page_height = $this->pdf->ez['pageHeight'];
		
		$this->current_justification = "left";
		$this->current_line_spacing = 1;
		$this->current_indent = 0;
		$this->current_font_size = 12;
		$this->underlined = false;
		$this->font_styles = array();
		$this->set_font($this->default_font_index);
		$this->list_levels = array();
		$this->page_numbering = false;
		$this->force_next_x = false;
		$this->force_next_width = false;
	}

	function document_end() {
		// har har har!
	}

	function get() {
		return $this->pdf->ezOutput();
	}
	
	function debug($s) {
		/*
		$fp = fopen("/tmp/test.log","a");
		if ($fp) {
			fwrite($fp,"$s\n");
			fclose($fp);
		}
		*/
	}

	function save($filename) {
		if (strpos($filename,".")===false) $filename .= ".pdf";
		$fp = fopen($filename,"w");
		if ($fp) {
			fwrite($fp,$this->pdf->ezOutput());
			fclose($fp);
		}
	}

	function get_mime_type() {
		return "application/pdf";	
	}
	
	function get_default_extension() {
		return "pdf";
	}
	
	function _check_font($name) {
		$filename = FONT_PATH.$name.".afm";
		return file_exists($filename);
	}

	function set_font($font_index) {
		$this->current_font = $font_index;
		$font = $this->font_list_table[$font_index];
		$translation_table = array(
			"arial"=>"Helvetica",
			"tahoma"=>"Helvetica",
			"verdana"=>"Helvetica",
			"times new roman"=>"Times",
			"courier new"=>"Courier",
		);
		
		
		$fontlower = strtolower($font);
		if ($translation_table[$fontlower]) $font = $translation_table[$fontlower];

		
		if (count($this->font_styles)) $font .= "-";
		if ($this->font_styles['bold']) {
			if ($this->_check_font($font."Bold")) $font .= "Bold";
		}
		if ($this->font_styles['italic']) {
			if ($this->_check_font($font."Italic")) {
				$font .= "Italic";
			} elseif ($this->_check_font($font."Oblique")) {
				$font .= "Oblique";
			}
		}
		
		if (substr($font,-1)=="-") $font = substr($font,0,strlen($font)-1);
		if ($font=="Times") $font = "Times-Roman";
		
		$filename = FONT_PATH.$font.".afm";
		
		if (!$this->_check_font($font)) {
			// helvetica is our fallback font.. it should (hopefully) always be there
			$filename = FONT_PATH."Helvetica.afm";
		}
		$this->pdf->selectFont($filename);
		$this->current_font_filename = $filename;
	}
	
	function set_font_size($font_size) {
		$this->current_font_size = $font_size;
	}

	function set_font_by_name($font_name) {
		$font_name = strtolower($font_name);
		reset($this->font_list_table);
		$k = array_search($font_name,$this->font_list_table);
		if ($k) $this->set_font($k);
	}
	
	function increment_indent() {
		$this->current_indent += $this->INDENT_INCREMENT;
	}

	function decrement_indent() {
		$this->current_indent -= $this->INDENT_INCREMENT;
	}
	
	function set_indent($inches) {
		$this->current_indent = $inches * 72;
	}
	
	function table_begin() {
		$this->text("\n");
	}

	function table_end() {
		$this->text("\n");
	}
	
	function image($filename,$width,$justification,$border,$padding) {
		if (!$width) $width = 0;
		if (!$justification) $justification = 'left';
		if (!$border) $border = 0;
		if (!$padding) $padding = 5;
		$this->pdf->ezImage($filename,$padding,$width,'none',$justification,$border);
	}
	
	function row_cell_begin($xpercent,$widthpercent) {
		$available_width = $this->page_width - $this->current_margins["right"] - $this->current_margins["left"];
		$this->force_next_x = round( $available_width * $xpercent / 100);
		$this->force_next_width = round($available_width * $widthpercent / 100);
		$this->pdf->y = $this->row_top_y;
		
		//echo "[X: {$this->force_next_x} @ {$xpercent}%][Cell width: {$this->force_next_width} @ {$widthpercent}%]<br>";
		
		//echo "[Setting next_text_x to {$this->pdf->next_text_x}]<br>";
	}
	
	function row_cell_end() {
		$this->row_max_y = max($this->pdf->y,$this->row_max_y);
	}
	
	function row_begin() {
		$this->row_top_y = $this->row_max_y = $this->pdf->y;
	}
	
	function row_end() {
		$this->pdf->y = $this->row_max_y;
		$this->text("\n");
	}
	
	function text($output_text) {

		$font_height_subtract = 0;
		if (substr($output_text,-1)!="\n") {
			// by default, ezText() appends a newline even if none was specified,
			// so we need to curb that behavior... do so here.

			$font_height_subtract = $this->pdf->getFontHeight($size) * $this->current_line_spacing;
		}

		if ($this->underlined) $output_text = "<u>$output_text</u>";

		if ($this->pdf->next_text_x==0 && $this->no_starting_blanks) {
			$output_text = preg_replace("/^([\n]*) /","\\1",$output_text);
		}

		$properties = array(
			"left"=>$this->current_indent,
			"spacing"=>$this->current_line_spacing,
			"justification"=>$this->current_justification
		);
		if ($this->force_next_x!==false) {
			unset($properties['left']);
			$properties['aleft'] = $this->current_margins['left'] + $this->force_next_x;
			$this->pdf->next_text_x = 0;
		}
		if ($this->force_next_width!==false) {
			$properties['aright'] = min(
				$this->current_margins["left"] + $this->force_next_x + $this->force_next_width,
				$this->page_width - $this->current_margins["right"]
			);
			//echo "Forcing X to: ".$properties['aleft']."<br>" . "Right margin is: ".$properties['aright']."<br>". "Page width is: ".$this->page_width."<br><br>";
		}

		$this->force_next_x = false;
		$this->force_next_width = false;
		
		$this->pdf->ezText(
			$output_text,
			$this->current_font_size,
			$properties
		);
	}
	
	function bold($activate) {
		if ($activate) $this->font_styles["bold"] = true; else unset($this->font_styles["bold"]);
		$this->set_font($this->current_font);
	}
	
	function italic($activate) {
		if ($activate) $this->font_styles["italic"] = true; else unset($this->font_styles["italic"]);
		$this->set_font($this->current_font);
	}
	
	function underline($activate) {
		$this->underlined = $activate;
	}
	
	function header_footer_content(&$content,$is_header) {
		$objheader = $this->pdf->openObject();
		$this->pdf->saveState();
		
		$pagenums = false;
		
		$regions = array("left","center","right");
		$font_changed = false;
		$size_changed = false;
		$this->pdf->setColor(0.75,0.75,0.75);
		$this->pdf->setStrokeColor(0.75,0.75,0.75);
		foreach ($regions as $region) {
			//echo "Region: $region<br>";
			$data = &$content[$region];
			

			if ($data["font"]) {
				$this->set_font($data["font"]);
				$font_changed = true;
			}
			if (!$data["font"]) {
				if ($this->current_font!=$this->default_font_index) $this->set_font($this->default_font_index);
			}
			if (!$data["size"]) $data["size"] = $this->current_font_size;
			
			if ($data["bold"]) $this->bold(true);
			if ($data["italic"]) $this->italic(true);
			
			$text = $data["text"];

			if ($data["underline"]) $text = "<c:uline>$text</c:uline>";

			if ($is_header) {
				$toppx = $this->page_height - $this->current_margins["top"] / 2;
			} else {
				$toppx = $this->current_margins["top"] / 2;
			}
			
			$pagenumbering = (strpos($text,"{PAGENUM}")!==false || strpos($text,"{TOTALPAGENUM}")!==false);
			if ($pagenumbering) {
				$text = str_replace(array("{PAGENUM}","{TOTALPAGENUM}"),"00",$text);
			}
			
			switch($region) {
				case "left":
					$leftpx = $this->current_margins["left"];
					break;
				case "center":
					if ($pagenumbering) {
						$leftpx = $this->page_width / 2;
						$this->debug("pagenumbering - setting leftpx to $leftpx");
					} else {
						$leftpx = ($this->page_width - $this->pdf->getTextWidth($data["size"],$text))  / 2;
					}
					break;
				case "right":
					$leftpx = $this->page_width - $this->current_margins["right"] - $this->pdf->getTextWidth($data["size"],$text);
					break;
			}
			
			if ($pagenumbering) {
				$pagenums = array(
					"left"=>$this->page_width - $this->current_margins["right"],
					"top"=>$toppx,
					"text"=>$data["text"],
					"size"=>$data["size"],
					"options"=>array(
						"color"=>array(0.75,0.75,0.75),
						"font"=>$this->current_font_filename
					)
				);
				switch($region) {
					case "left": $pagenums["location"] = "right"; break;
					case "right": $pagenums["location"] = "left"; break;
					case "center": $pagenums["location"] = "center"; break;
				}
				$this->debug("Pagenums set");
				
			} else {
				$this->pdf->addText($leftpx,$toppx,$data["size"],$text);
			}
			
			if ($data["italic"]) $this->italic(false);
			if ($data["bold"]) $this->bold(false);
		}
		
		$this->pdf->restoreState();
		$this->pdf->closeObject();
		$this->pdf->addObject($objheader,'all');

		if ($pagenums) {
			//setNum = ezStartPageNumbers(x,y,size,[pos],[pattern],[num])			
			
			$this->pdf->ezStartPageNumbers(
				$pagenums["left"],
				$pagenums["top"],
				$pagenums["size"],
				$pagenums["location"],
				$pagenums["text"],
				1,
				$pagenums["options"]
			);
			$this->page_numbering = true;
			$this->debug("Starting page numbering (".$pagenums["left"]."x".$pagenums["top"].", as ".$pagenums["text"].", font size ".$pagenums["size"].")");
		}
		
	}
	
	function header($content) {
		$this->header_footer_content($content,true);
	}
	
	function footer($content) {
		$this->header_footer_content($content,false);
	}

	function set_margins($left,$right,$top,$bottom) {
		$margins = array("left","right","top","bottom");
		foreach ($margins as $k=>$margin) {
			$$margin = $$margin * 72;
			$this->current_margins[$margin] = $$margin;
		}
		$this->pdf->ezSetMargins($top,$bottom,$left,$right);
	}
	
	function line_spacing($value) { // set to 1, 1.5, 2, etc.
		$this->current_line_spacing = $value;
	}
	
	function page_break() {
		$this->pdf->ezNewPage();
	}

	function section_break() {
		$this->pdf->ezNewPage();
	}
	
	function list_begin($list_style="numeric") {
		if (is_null($list_format)) {
			switch($list_style) {
				case "alphalower":
				case "alphaupper":
					$list_format = "%s.";
					break;
				case "bulleted":
					$list_format = "%s";
					break;
				default:
					$list_format = "%d.";
					break;
			}
		}
		$list_level = array(
			"style"=>$list_style,
			"index"=>0,
			"format"=>$list_format,
		);
		array_unshift($this->list_levels,$list_level);
		$this->increment_indent();
	}
	
	function list_end() {
		$list_level = array_shift($this->list_levels);
		if ($list_level) $this->decrement_indent();
	}
	
	function list_item($item) {
		$list_level = &$this->list_levels[0];

		$list_level["index"]++;
		$index = $list_level["index"];
		switch($list_level["style"]) {
			case "numeric":
				$bullet = $index; break;
			case "alphalower":
				$bullet = chr(ord('a')+$index-1); break;
			case "alphaupper":
				$bullet = chr(ord('A')+$index-1); break;
			case "bulleted":
				$bullet = "-"; break;
		}
	

		$currenty = $this->pdf->ezText(
			sprintf($list_level["format"],$bullet),
			$this->current_font_size,
			array(
				"left"=>$this->current_indent,
				"spacing"=>$this->current_line_spacing,
				"justification"=>$this->current_justification
			)
		);
		
	    $height = $this->pdf->getFontHeight($this->current_font_size) * $this->current_line_spacing;

		$this->pdf->addText(
			$this->current_margins["left"]+$this->current_indent+$this->INDENT_INCREMENT,
			$currenty - $height,
			$this->current_font_size,
			$item
		);
		
		$this->text("\n");
	}
	
	function align($alignment) {
		$this->current_justification = $alignment;
	}
	
	
	function set_color($r,$g,$b) {
		$this->pdf->setColor($r,$g,$b);
		$this->pdf->setStrokeColor($r,$g,$b);
	}
	

}
?>