<?php
/* HTMLConverter - Simple HTML to PDF/RTF/TXT/etc converter
 * Copyright 2004-2006, Steve Blinch
 * http://code.blitzaffe.com
 * ============================================================================
 *
 * LICENSE
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */ 
require_once(dirname(__FILE__)."/class_RTF.php");
require_once(dirname(__FILE__)."/class_DocumentMaker.php");

class RTFMaker extends DocumentMaker {
	
	function RTFMaker() {
		$this->rtf = &new RTF();
	}
	
	function generator_options($options) {
	}

	function document_begin($orientation="portrait",$paperformat="a4") {
		$this->rtf->INDENT_INCREMENT = floor(1440 / floor(1 / $this->indent_inches));
		$this->rtf->start($this->font_list_table,$this->default_font_index,$this->default_font_size,$orientation=="landscape");
		
	}

	function document_end() {
		$this->rtf->finish();
		
		if ($this->no_starting_blanks) {
			$this->rtf->rtf_output = preg_replace("/(\\\\par[\S]*)([\s]{2,})/","\\1 ",$this->rtf->rtf_output);
		}
		
	}

	function get() {
		return $this->rtf->get();
	}

	function save($filename) {
		if (strpos($filename,".")===false) $filename .= ".rtf";
		$this->rtf->save($filename);
	}

	function get_mime_type() {
		return "application/rtf";
	}
	
	function get_default_extension() {
		return "rtf";
	}

	function set_font($font_index) {
		$this->rtf->set_font($font_index);
	}
	
	function set_font_size($font_size) {
		$this->rtf->set_font_size($font_size);
	}

	function set_font_by_name($font_name) {
		$font_name = strtolower($font_name);
		reset($this->font_list_table);
		$k = array_search($font_name,$this->font_list_table);
		if ($k) $this->set_font($k);
	}
	
	function increment_indent() {
		$this->rtf->inc_indent();
	}

	function decrement_indent() {
		$this->rtf->dec_indent();
	}
	
	function set_indent($inches) {
		$indent = floor($inches * 1440);
		$this->rtf->left_indent = $indent;
		$this->rtf->tab_indent = $indent;
		$this->rtf->set_indent($this->rtf->first_indent,$this->rtf->left_indent,$this->rtf->tab_indent);
	}

	function text($output_text) {
		$output_text = str_replace("\n","\\par",$output_text);
		$output_text = preg_replace("/\\\\par([^\\\\{])/","\\par \\1",$output_text);
		$this->rtf->text($output_text);
	}
	
	function bold($activate) {
		$this->rtf->bold($activate);
	}
	
	function italic($activate) {
		$this->rtf->italic($activate);
	}
	
	function underline($activate) {
		$this->rtf->underline($activate);
	}
	
	function header_footer_content(&$content) {
		$tab_width = floor(($this->rtf->page_width-$this->rtf->margins["left"]-$this->rtf->margins["right"]) / 3);

		$this->rtf->set_tab_stop(0);
		$this->rtf->set_tab("center");
		$this->rtf->set_tab_stop($tab_width+$this->rtf->margins["left"]);
		$this->rtf->set_tab("right");
		$this->rtf->set_tab_stop($this->rtf->page_width-$this->rtf->margins["left"]-$this->rtf->margins["right"]);
		
		
		$regions = array("left","center","right");
		$font_changed = false;
		$size_changed = false;
		foreach ($regions as $region) {
			$data = &$content[$region];
			
			if (!$data["font"] && $font_changed) $data["font"] = $this->default_font_index;
			if (!$data["size"] && $size_changed) $data["size"] = $this->default_font_size;

			if ($data["font"]) {
				$this->rtf->set_font($data["font"]);
				$font_changed = true;
			}
			if ($data["size"]) {
				$this->rtf->set_font_size($data["size"]);
				$size_changed = true;
			}
			
			if ($data["bold"]) $this->rtf->bold(true);
			if ($data["italic"]) $this->rtf->italic(true);
			if ($data["underline"]) $this->rtf->underline(true);
			
			$text = $data["text"];
			
			$text = str_replace(
				array("{PAGENUM}","{TOTALPAGENUM}"),
				array($this->rtf->get_page_number(),$this->rtf->get_page_count()),
				$text
			);

			$this->rtf->text($text);
			if ($data["underline"]) $this->rtf->underline(false);
			if ($data["italic"]) $this->rtf->italic(false);
			if ($data["bold"]) $this->rtf->bold(false);
			
			if ($region!="right") $this->rtf->tab();
		}
		
		$this->rtf->paragraph();
	}
	
	function header($content) {
		$this->rtf->header_start();
		$this->header_footer_content($content);
		$this->rtf->header_finish();
	}
	
	function footer($content) {
		$this->rtf->footer_start();
		$this->header_footer_content($content);
		$this->rtf->footer_end();
	}

	function set_margins($left,$right,$top,$bottom) {
		$this->rtf->set_margins($left*1440,$right*1440,$top*1440,$bottom*1440);
	}
	
	function line_spacing($value) { // set to 1, 1.5, 2, etc.
		// rough-ass, very poor estimate based on 12-pt font size
		$spacing = floor(11.2 * $current_font_size);
		$this->rtf->line_spacing($spacing);
	}
	
	function image($filename) {
		// We're not going to screw around with RTF images, thanks.
	}

	function page_break() {
		$this->rtf->page_break();
	}

	function section_break() {
		$this->rtf->section_break();
	}
	
	function list_begin($list_style="numeric") {
		$this->rtf->list_start($list_style);
	}
	
	function list_end() {
		$this->rtf->list_finish();
	}
	
	function list_item($item) {
		$this->rtf->list_item($item);
	}
	
	function align($alignment) {
		$this->rtf->set_align($alignment);
	}	

	function set_color($r,$g,$b) {
		$r *= 255; $g *= 255; $b *= 255;
		//echo "[$r/$g/$b]";
		$this->rtf->set_color($r,$g,$b);
	}


}
?>