<?php
/* HTMLConverter - Simple HTML to PDF/RTF/TXT/etc converter
 * Copyright 2004-2006, Steve Blinch
 * http://code.blitzaffe.com
 * ============================================================================
 *
 * LICENSE
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */ 
require_once(dirname(__FILE__)."/class_DocumentMaker.php");

class TXTMaker extends DocumentMaker {
	
	function TXTMaker() {
	}
	
	function generator_options($options) {
	}

	function document_begin($orientation="portrait",$paperformat="a4") {
		$this->INDENT_INCREMENT = 4;
		$this->current_indent = 0;
		$this->list_levels = array();
		$this->output = "";
		$this->line_spacing = 1;
		
		$this->last_line = "";
		$this->page_line = 0;
		$this->page_number = 1;
		
		// number of columns used for left/right margins (per "inch")
		$this->MARGIN_COLUMNS = $this->INDENT_INCREMENT;
		// number of lines used for top/bottom margins (per "inch");
		$this->MARGIN_LINES = 3;
		
		
		if ($orientation=="landscape") {
			$this->wrap_col = 120;
			$this->page_lines = 90;
		} else {
			$this->wrap_col = 80;
			$this->page_lines = 40; //100;
		}

		$this->output .= "\n";
		$this->line_break = "\r\n";
	}
	
	function header_footer_content(&$content,$is_header,$page_num,$total_pages) {
		$regions = array("left","center","right");
		$line = str_repeat(" ",$this->wrap_col);
		foreach ($regions as $region) {
			$data = &$content[$region];
			
			$text = $data["text"];
			
			$text = str_replace(
				array("{PAGENUM}","{TOTALPAGENUM}"),
				array($page_num,$total_pages),
				$text
			);
			
			switch($region) {
				case "left": $start = 0; break;
				case "center": $start = floor(($this->wrap_col - strlen($text)) / 2); break;
				case "right": $start = $this->wrap_col - strlen($text); break;
			}
			$line = substr_replace($line,$text,$start,strlen($text));
		}
		return $line;
	}	
	
	function get_header() {
		return "\r\n{%~%HEADER%~%}\r\n".str_repeat("-",$this->wrap_col)."\r\n";
	}

	function get_footer() {
		return "\r\n\r\n".str_repeat("-",$this->wrap_col)."\r\n{%~%FOOTER%~%}\r\n";
	}

	function replace_header($x) {
		$content = $this->header_footer_content($this->header_content,true,$this->footer_page_no,$this->total_pages);
		
		$this->header_page_no++;
		return $content;
	}

	function replace_footer($x) {
		$content = $this->header_footer_content($this->footer_content,false,$this->footer_page_no,$this->total_pages);
		
		$this->footer_page_no++;
		return $content;
	}

	function document_end() {
		if ($this->current_line) $this->text("",true);

		$pageno = 1;
		
		if ($this->header_content || $this->footer_content) {
			$lines = explode("\r\n",$this->output);
			$lineno = 0;
			foreach ($lines as $id=>$line) {
				
				if (trim($line)=="{%~%PAGE_BREAK%~%}") {
					$lines[$id] = str_repeat("\r\n",$this->page_lines-$lineno-2);
					$lineno = $this->page_lines-1;
				}
				if ($this->no_starting_blanks) {
					$lines[$id] = preg_replace("/^ /","\\1",$lines[$id]);
				}

				if ($lineno == 0) {
					if ($this->header_content) $lines[$id] =  $this->get_header($pageno) . $lines[$id];
				}
				$lineno++;
				
				if ($lineno == $this->page_lines) {
					if ($this->footer_content) $lines[$id] .= $this->get_footer($pageno);
					$pageno++;
					$lineno = 0;
				}
			}
			
			if ($lineno) {
				if ($this->footer_content) $lines[count($lines)-1] .= str_repeat("\r\n",$this->page_lines-$lineno).$this->get_footer($pageno);
			}

			$this->output = implode("\r\n",$lines);
		} else {
			$this->output = str_replace("{%~%PAGE_BREAK%~%}","\r\n",$this->output);
		}
		
		$this->header_page_no = 1;
		$this->footer_page_no = 1;
		$this->total_pages = $pageno;
		
		$this->output = preg_replace_callback("/{%~%FOOTER%~%}/",array(&$this,"replace_footer"),$this->output);
		$this->output = preg_replace_callback("/{%~%HEADER%~%}/",array(&$this,"replace_header"),$this->output);
		
		$this->output = trim($this->output);
	}

	function get() {
		return $this->output;
	}

	function save($filename) {
		if (strpos($filename,".")===false) $filename .= ".txt";
		$fp = fopen($filename,"w");
		if ($fp) {
			fwrite($fp,$this->output);
			fclose($fp);
		}
	}

	function get_mime_type() {
		return "text/plain";
	}
	
	function get_default_extension() {
		return "txt";
	}

	function set_font($font_index) {
		// no font support, obviously
	}

	function set_font_size($font_size) {
	}

	function increment_indent() {
		$this->current_indent += $this->INDENT_INCREMENT;
		
	}

	function decrement_indent() {
		$this->current_indent -= $this->INDENT_INCREMENT;
	}
	
	function set_indent($inches) {
		$this->current_indent = floor($this->INDENT_INCREMENT * $inches);
	}
	
	function next_token(&$output_text) {
		if (substr($output_text,0,1)=="\n") return "\n";
		$p = strpos($output_text,' ');
		if ($p!==false) {
			$nl = strpos($output_text,"\n");
			if ($nl!==false) {
				$nextbreak = ($p<$nl?$p:$nl);
				$isspace = $p<$nl;
			} else {
				$nextbreak = $p;
				$isspace = true;
			}
		} else {
			$nl = strpos($output_text,"\n");
			if ($nl!==false) {
				$nextbreak = $nl;
				$isspace = false;
			} else {
				$nextbreak = false;
			}
		}
		if ($nextbreak === false) {
			if ($output_text) {
				return $output_text;
			} else {
				return false;
			}
		}

		if ($isspace) $nextbreak++;
		return substr($output_text,0,$nextbreak);
	}
	
	function store_current_line($leftmargincols,$wrapcol) {
		//echo "[$wrapcol][".strlen($this->current_line)."][{$this->current_line}]<br>";
		
		$line = &$this->current_line;
		if (trim($line)) {
			switch($this->alignment) {
				case "right":
					$line = str_pad($line,$wrapcol," ",STR_PAD_LEFT);
					break;
				case "center":
					$padleft = floor(($wrapcol - strlen($line)) / 2);
					$line = str_repeat(" ",$padleft).$line;
					break;
				default:
					break;
			}
			$line = str_repeat(" ",$this->current_indent+$leftmargincols).$line;
		}		
		$this->output .= $line . $this->line_break;
		$this->current_line = "";
	}
	
	function text($output_text,$flush=false) {
		//echo "[$output_text]\n";
		$leftmargincols = ($this->margin["left"]-1)*$this->MARGIN_COLUMNS;
		$wrapcol = 
			$this->wrap_col - 
			$leftmargincols - 
			($this->margin["right"]-1)*$this->MARGIN_COLUMNS -
			$this->current_indent;
		
		if ($flush) {
			$this->store_current_line($leftmargincols,$wrapcol);
			return;
		}

		$its = 0;
		while (true) {
			$its++;
			if ($its==100000) {
				$this->output .= "-- (spiraling out of control, breaking) --".$this->line_break;
				return;
			}

			$tok = $this->next_token($output_text);
			//echo "[$tok]";
			if ($tok===false) {
				return;
			}

			if ($tok=="\n") {
				$this->store_current_line($leftmargincols,$wrapcol);
				$output_text = substr($output_text,1);
				continue;
			}
			
			if (strlen($tok)+strlen($this->current_line)<=$wrapcol) {
				$this->current_line .= $tok;
				$output_text = substr($output_text,strlen($tok));
			} else {
				$this->store_current_line($leftmargincols,$wrapcol);
				//break;
			}
		}
		return;
/*		
		
//		$output_text = wordwrap($output_text,$wrapcol,"\n",1);

		$txt = explode("\n",$output_text);

		foreach ($txt as $id=>$line) {
			//if (substr($line,-1)=="\n") $line = substr($line,0,strlen($line)-1);
			//if (substr($line,-1)=="\r") $line = substr($line,0,strlen($line)-1);
			//$line = rtrim($line,"\r\n\t");
			
			if ($line) {
				switch($this->alignment) {
					case "right":
						$line = str_pad($line,$wrapcol," ",STR_PAD_LEFT);
						break;
					case "center":
						$padleft = floor(($wrapcol - strlen($line)) / 2);
						$line = str_repeat(" ",$padleft).$line;
						break;
					default:
						break;
				}
				$line = str_repeat(" ",$this->current_indent+$leftmargincols).$line;
			}
			
			$txt[$id] = $line;
		}
		
		$this->current_line_len = strlen($line);

		$this->output .= implode($this->line_break,$txt);
		*/
	}
	
	
	function bold($activate) {
	}
	
	function italic($activate) {
	}
	
	function underline($activate) {
	}
	
	
	function header($content) {
		$this->header_content = $content;
	}
	
	function footer($content) {
		$this->footer_content = $content;
	}

	function set_margins($left,$right,$top,$bottom) {
		foreach (array("left","right","top","bottom") as $k=>$side) $this->margin[$side] = $$side;
	}
	
	function line_spacing($value) { // set to 1, 1.5, 2, etc.
		$this->line_spacing = floor($value);
		$this->line_break = str_repeat("\r\n",$this->line_spacing);
	}
	
	function image($filename) {
	}

	function page_break() {
		$this->text("\n{%~%PAGE_BREAK%~%}\n");
	}

	function section_break() {
		$this->page_break();
	}
	
	function list_begin($list_style="numeric") {
		if (is_null($list_format)) {
			switch($list_style) {
				case "alphalower":
				case "alphaupper":
					$list_format = "%s.";
					break;
				case "bulleted":
					$list_format = "%s";
					break;
				default:
					$list_format = "%d.";
					break;
			}
		}
		$list_level = array(
			"style"=>$list_style,
			"index"=>0,
			"format"=>$list_format,
		);
		array_unshift($this->list_levels,$list_level);
		$this->increment_indent();
	}
	
	function list_end() {
		$list_level = array_shift($this->list_levels);
		if ($list_level) $this->decrement_indent();
	}
	
	function list_item($item) {
		$list_level = &$this->list_levels[0];

		$list_level["index"]++;
		$index = $list_level["index"];
		switch($list_level["style"]) {
			case "numeric":
				$bullet = $index; break;
			case "alphalower":
				$bullet = chr(ord('a')+$index-1); break;
			case "alphaupper":
				$bullet = chr(ord('A')+$index-1); break;
			case "bulleted":
				$bullet = "-"; break;
		}

		$this->text(
			str_pad(
				sprintf($list_level["format"],$bullet)." ",
				$this->INDENT_INCREMENT,
				" ",STR_PAD_RIGHT
			).
			$item."\n"
		);
	}
	
	function align($alignment) {
		$this->alignment = $alignment;
		// TODO: implement this
		//$this->rtf->set_align($alignment);
	}	

}
?>